/* 
 * File:   main.c
 * Author: VEL255
 *
 * Created on 15/11/2012
 *
 **********************************************************************
 * C Compiler Demo for the EDU10 Board.
 * This code uses the PIC16F882 ADC to read the value of the VDR, NTC and RV1.
 * Operation modes:
 *
 * RESET:
 * LDR is programmed as light detector.
 * LDR turns LED LD8 on/off depending on light.
 * Potentiometer RV1 changes display brightness.
 * If NTC voltage is <1V or >4V display "E" (Error).
 *
 * SW1 pressed:
 * Turn on LD7
 * Display 7
 *
 * SW2 pressed:
 * Turn on LD6
 * Display 6
 *
 * SW3 pressed:
 * Turn on LD5
 * Display 5
 *
 * SW4 pressed:
 * Turn on LD4
 * Buzzer beep
 * Display all segments+dot
**********************************************************************/

//#include <stdio.h>
//#include <stdlib.h>
//#include <stdint.h>
#include <xc.h>

//------------------------------------------------------------------------------
//PRAGMA's

#pragma config  CPD =       OFF
#pragma config  BOREN =     OFF
#pragma config  IESO =      OFF
#pragma config  DEBUG =     OFF
#pragma config  FOSC =      INTRC_NOCLKOUT
#pragma config  FCMEN =     OFF
#pragma config  MCLRE =     ON
#pragma config  WDTE =      OFF
#pragma config  CP =        OFF
#pragma config  LVP =       OFF
#pragma config  PWRTE =     OFF
#pragma config  BOR4V =     BOR21V
#pragma config  WRT =       OFF

//------------------------------------------------------------------------------
//DEFINES

#define LD7     PORTCbits.RC7
#define LD6     PORTCbits.RC6
#define LD5     PORTCbits.RC5
#define LD4     PORTCbits.RC4
#define buzzer  PORTCbits.RC0
#define offLD   PORTC &= 0b00001111

#define SW1     PORTAbits.RA4
#define SW2     PORTAbits.RA5
#define SW3     PORTAbits.RA6
#define SW4     PORTAbits.RA7

//------------------------------------------------------------------------------
//DECLARATIONS

void init (void);
void beep (void);
void idle (void);
void delay (void);

const unsigned char segments7[] = {
    0b00111111, // 0
    0b00000110, // 1
    0b01011011, // 2
    0b01001111, // 3
    0b01100110, // 4
    0b01101101, // 5
    0b01111101, // 6
    0b00000111, // 7
    0b01111111, // 8
    0b01101111, // 9
    0b01110111, // A
    0b01111100, // b
    0b01011000, // c
    0b01011110, // d
    0b01111001, // E
    0b01110001, // F
    0b11111111, // 8.
    0b00000000  // off
};

//------------------------------------------------------------------------------
//CODE

//------------------------------------------------------------------------------
void main (void)
{
    init();
    while(1)
    {
        if (SW1 == 1)
        {
            offLD;
            LD7 = 1;
            PORTB = segments7[7];
        }
        else if (SW2 == 1)
        {
            offLD;
            LD6 = 1;
            PORTB = segments7[6];
        }
        else if (SW3 == 1)
        {
            offLD;
            LD5 = 1;
            PORTB = segments7[5];
        }
        else if (SW4 == 1)
        {
            offLD;
            LD4 = 1;
            PORTB = segments7[16];
            beep();
            PORTC = 0x00;
        }
        if (!(PORTC & 0b11110000))   // SW4 pressed or no button pressed
            idle();
    }
}
//------------------------------------------------------------------------------
void idle (void)
{
    unsigned char displayOn, displayOff;

    ADCON0 = 0b10000001;    // Analog-to-Digital FOSC/32, Channel 0 (RA0),
                            //   input from NTC
    delay();                // Acquisition delay 10us
    ADCON0bits.GO = 1;      // start conversion
    while (ADCON0bits.GO);  // wait until done
    if (ADRESH>204 || ADRESH<51)
        PORTB = segments7[14];  // error message 'E'
    else
    {
        ADCON0 = 0b10000101;  	// Analog-to-Digital FOSC/32, Channel 1 (RA1),
                                //   input from   LDR
        delay();                // Acquisition delay 10us
        ADCON0bits.GO = 1;      // start conversion
        while (ADCON0bits.GO);  // wait until done
        ADRESH &= 0b10000000;   // mask the MSb from the result
        ADRESH >>= 2;           // divide by 4 to dim the bright LED
        CCPR1L = ADRESH;        // PWM output to bright LED

        ADCON0 = 0b10001001;     // Analog-to-Digital FOSC/32, Channel 2 (RA2),
                                 //   input from  RV1
        delay();                 // Acquisition delay 10us
        ADCON0bits.GO = 1;       // start conversion
        while (ADCON0bits.GO);   // wait until done
        displayOn = ADRESH;
        displayOff = 255 - ADRESH;
        PORTB = segments7[0];       // display '0' a short period of time
        while (displayOn--)
        {
            PORTB = segments7[0];   // display '0'
            delay();
        }
        while (displayOff--)
        {
            PORTB = segments7[17];  // display off
            delay();
        }
    }
}

//------------------------------------------------------------------------------
void delay (void)
{
    unsigned char i = 1;
    while (i--);                    // delay about 10us @ osc 8MHz
}

//------------------------------------------------------------------------------
void beep (void)
{
    unsigned char i, j, c;
    i = 10;                         // total beep duration
    while (i--)
    {
        j = 100;
        while (j--)
        {
            buzzer ^= 1;            // invert buzzer bit
            c = 100;                // first beep pitch
            while (c--);            
        }
        j = 100;
        while (j--)
        {
            buzzer ^= 1;            // invert buzzer bit
            c = 110;                // second beep pitch
            while (c--);            
        }
    }
}

//------------------------------------------------------------------------------
void init(void)
{
    //OSC
    OSCCONbits.IRCF0 = 1;        // internal osc = 8Mhz
    OSCCONbits.IRCF1 = 1;
    OSCCONbits.IRCF2 = 1;
    OSCCONbits.OSTS = 1;        // osc is selected by FOSC (pragma)
    OSCCONbits.SCS = 0;         // clock source defined by FOSC (pragma)

    //PORTA
    PORTA = 0b00000000;         // clear PORTA
    ANSEL = 0b00000111;         // 0-2 PORTA pins Analog
    TRISA = 0b11111111;         // all pins are input

    //PORTB
    PORTB = 0b00000000;         // clear PORTB
    TRISB = 0b00000000;         // all pins are output (7 SEGMENT)
    WPUB = 0b00000000;          // weak pull-ups disabled
    IOCB = 0b00000000;          // interrupt on change disabled

    //PORTC
    PORTC = 0b00000000;         // clear PORTC
    TRISC = 0b00000000;         // all pins are output

    //INITS
    CCP1CON = 0b00001100;       // enable PWM mode
    T2CON = 0b00000100;         // timer2 on, 1:1 prescale
    CCPR1L = 0x00;		// clear Duty Cycle bits 9 - 2
}

